#
#   5B - Controls
#

from pygame import Rect
from albow.widget import Widget
from albow.controls import Label, Button, TextField, IntField, Row, Column
from albow.dialogs import Dialog

#----------------------------------------------------------------------------

class TextDisplay(Widget):

	#fg_color = (255, 255, 255)
	border_width = 1
	margin = 3

	def __init__(self, width, client, attr, click_action = None, **kwds):
		Widget.__init__(self, **kwds)
		self.set_size_for_text(width)
		self.client = client
		self.attr = attr
		if click_action:
			self.click_action = click_action

	def draw(self, surf):
		frame = self.get_margin_rect()
		text = self.get_text()
		buf = self.font.render(text, True, self.fg_color)
		r = buf.get_rect()
		r.midleft = frame.midleft
		surf.blit(buf, r)
	
	def get_text(self):
		return getattr(self.client, self.attr)
	
	def mouse_down(self, e):
		self.call_handler('click_action')

#----------------------------------------------------------------------------

class TimeDisplay(TextDisplay):

	def __init__(self, client, attr, **kwds):
		TextDisplay.__init__(self, "12:59", client, attr,  **kwds)
	
	def get_text(self):
		return time_to_text(TextDisplay.get_text(self))

#----------------------------------------------------------------------------

def text_to_time(x):
	if not x:
		return -1
	else:
		t = "".join([c for c in x if c in ":0123456789"])
		if ":" not in t:
			t = ":" + t
		m, s = t.split(":")[:2]
		def i(v):
			return int(v or "0")
		return i(m) * 60 + i(s)

def time_to_text(secs):
	if secs < 0:
		return ""
	else:
		return "%d:%02d" % (secs // 60, secs % 60)

#----------------------------------------------------------------------------

class Form(Dialog):

	#border_width = 1

	def __init__(self, *args, **kwds):
		Dialog.__init__(self, *args, **kwds)
		self.labels = []
		self.fields = []
		self.fields_by_name = {}
	
	def text_field(self, name, caption, *args, **kwds):
		self.add_field(name, caption, TextField(*args, **kwds))
	
	def int_field(self, name, caption, width = 50, *args, **kwds):
		self.add_field(name, caption, IntField(width, *args, **kwds))
	
	def add_field(self, name, caption, field):
		self.labels.append(Label(caption))
		self.fields.append(field)
		self.fields_by_name[name] = field
	
	def end(self):
		d = self.margin
		labels = Column(self.labels, align = 'l')
		fields = Column(self.fields, align = 'l')
		labels_and_fields = Row([labels, fields])
		buttons = Row([
			Button("OK", action = self.ok),
			Button("Cancel", action = self.cancel)
		], spacing = d)
		all = Column([labels_and_fields, buttons], spacing = d, align = 'r')
		all.rect.topleft = (d, d)
		self.add(all)
		self.shrink_wrap()
		if self.fields:
			self.fields[0].focus()
	
	def validate(self):
		for i, field in enumerate(self.fields):
			try:
				field.get_value()
			except ValueError, e:
				field.focus()
				alert("%s: %s" % (self.labels[i].text, e))
				return False
		return True

	def ok(self):
		if self.validate():
			self.dismiss(True)
	
	def cancel(self):
		self.dismiss(False)

	def __getitem__(self, name):
		return self.fields_by_name[name].get_value()

	def __setitem__(self, name, value):
		self.fields_by_name[name].set_value(value)
