#
#   5B - Relay
#

from albow.resource import get_sound
from albow.image_array import get_image_array
from image import RotatableImage
from part import Part
from controls import Form
from vector_algebra import Point2

sound_enabled = True

IDLE = 'idle'
OPERATING = 'operating'
RELEASING = 'releasing'

class Relay(Part):

	terminal_points = [(-20, 25), (-10, 25), (0, 25), (10, 25), (20, 25)]
	slug_points = [(-12, -10), (-12, 13)] # operate, release
	base_images = get_image_array("parts/relay.png", 2, border = 2)
	images = [RotatableImage(base_images[i]) for i in xrange(2)]
	slug_image = RotatableImage("parts/relay_slug.png")
	if sound_enabled:
		sounds = [
			get_sound("relay_pullin.ogg"),
			get_sound("relay_dropout.ogg")
		]
	is_rotatable = True
	
	delays = [0, 0] # operate, release
	operated = False
	
	def init_from_catalog(self, operate_delay, release_delay):
		self.delays = [operate_delay, release_delay]

	def reset(self):
		self.operated = False
		self.delay = 0
		
	def draw(self, surf, t):
		Part.draw(self, surf, t)
		tt = None
		for i, d in enumerate(self.delays):
			if d:
				p = Point2(self.slug_points[i])
				tt = t * self.local_to_global_transformation()
				self.slug_image.draw(surf, tt * p, self.orientation)

	def get_rotatable_image(self):
		return self.images[self.operated]

	def begin_frame(self):
		operated = self.operated
		powered = self.power_across(0, 1)
		if operated <> powered:
			self.delay += 1
			if self.delay > self.delays[operated]:
				if sound_enabled:
					self.sounds[operated].play()
				self.operated = powered
				self.delay = 0
		else:
			self.delay = 0

	def node_changed(self, n):
		if self.operated:
			m = 4
		else:
			m = 2
		if n == 3 or n == m:
			self.propagate(3, m)

	def get_params(self, params):
		params.extend(self.delays)
	
	def read_params(self, r):
		self.delays = [r.get_arg(int, 0), r.get_arg(int, 0)]

	def edit_params(self):
		form = Form()
		form.int_field('operate', "Operate delay")
		form.int_field('release', "Release delay")
		form.end()
		form['operate'], form['release'] = self.delays
		if form.present():
			self.delays = [form['operate'], form['release']]
			return True
		else:
			return False
